module Upman
  module Core
    module Config
      extend self
      using SymbolizeHelper

      @_config = {}
      attr_reader :_config

      def load!(filename, options = {})
        newsets = YAML::load_file(filename).deep_symbolize_keys
        newsets = newsets[options[:env].to_sym] if options[:env] && newsets[options[:env].to_sym]
        deep_merge!(@_config, newsets)
      end

      def get!(key)
        return @_config[key]
      end

      # Deep merging of hashes
      # deep_merge by Stefan Rusterholz, see http://www.ruby-forum.com/topic/142809
      def deep_merge!(target, data)
        merger = proc {|key, v1, v2|
          Hash === v1 && Hash === v2 ? v1.merge(v2, &merger) : v2}
        target.merge! data, &merger
      end

      def method_missing(name, *args, &block)
        @_config[name.to_sym] ||
            fail(NoMethodError, "unknown configuration root #{name}", caller)
      end
    end
  end
end
