module Upman
  module Core
    class Worker

      include Upman::Utils::Helper
      include Upman::Utils::Dynload

      attr_accessor :workers, :worker_threads

      def initialize
        @stop_signal = true
        @config = ::Upman::Core::Config.daemon
        @workers = []
        @worker_threads = []

        register_worker
      end

      def run!
        show_banner

        @workers.each do |worker|
          @worker_threads << Thread.new do
            worker.run!
          end
        end
        @worker_threads.each { |t| t.join }
      end

      def shutdown
        info "Stopping Worker Threads"
        @stop_signal = false
      end


      def register_worker
        @config[:workers].each do |worker|
          require_relative "../../upman/worker/#{worker}"
          ext_obj  = dynload("Upman::Worker::#{worker.split('_').map(&:capitalize).join('')}")
          info "Register Worker Thread Upman::Worker::#{worker.split('_').map(&:capitalize).join('')}"
          ext_class = ext_obj .new
          @workers.append ext_class.register
        end
      end


      private

      def show_banner
        info "Running UpMan #{::Upman::Version::VERSION}"
      end

    end
  end
end