# frozen_string_literal: true

require 'json'

module Upman
  module Extensions
    # Get install history on debian system
    class GetInstallHistory < ::Upman::Core::ExtensionBase
      REQUEST_PATH = 'getInstallHistory'

      def register
        Servlet
      end

      # Servlet action for WEBrick
      class Servlet < ::Upman::Server::BaseServlet

        attr_accessor :param_since

        # rubocop:disable Naming/MethodName
        def do_GET(request, response)
          # rubocop:enable Naming/MethodName

          super(request, response)

          unless is_authenticated(request)
            response = not_authorized(response, "Request declined")
            return response
          end

          unless (@param_since = get_param('since', 'date', false))
            return nil
          end

          if (body = perform_action)
            response = ok(response, body)
          end
        end

        def perform_action
          install_history_service = ::Upman::Service::InstallHistory.new
          install_history_service.get(@param_since).to_json
        end

      end
    end
  end
end
