module Upman
  module Server
    class BaseServlet < WEBrick::HTTPServlet::AbstractServlet

      include ::Upman::Utils::Helper

      attr_accessor :query, :response, :error_msg

      def initialize(server, *options)
        super(server, *options)
      end

      # rubocop:disable Naming/MethodName
      def do_GET(_request, response)
        # rubocop:enable Naming/MethodName

        info "Process API request #{_request.path}"

        @response = response
        @query = _request.query
      end

      def is_authenticated(request)
        node_service = Upman::Service::Node.new
        node_uuid = node_service.get_node_uuuid

        info "Checking Authentication"
        return true if request["x-upman-token"] == node_uuid

        fail "Access denied"

        false
      end

      def ok(response, body)
        response.status = 200
        response['Content-Type'] = 'application/json'
        response.body = body
        response
      end

      def not_authorized(response, body)
        response.status = 401
        response['Content-Type'] = 'application/json'
        response.body = "{\"status\": \"not_authorized\", \"message\": \"#{body}\"}"
        response
      end

      def bad_request(response, body)
        response.status = 400
        response['Content-Type'] = 'application/json'
        response.body = "{\"status\": \"bad_request\", \"message\": \"#{body}\"}"
        response
      end

      def get_param(key, validate, require = false)
        if @query.nil?
          return ''
        end

        unless @query.include? key
          unless require
            return ''
          else
            return nil
          end
        end
        unless validate.nil?
          if validate == 'date'
            begin
              @query[key] = DateTime.parse(@query[key])
            rescue
              msg = "Invalid content for param #{key}, expected type: date - found #{@query[key]}"
              bad_request(@response, msg)
              fail msg
              return false
            end
          end
        end
        @query[key]
      end
    end
  end
end