require 'openssl'
require 'webrick'
require 'webrick/https'

module Upman
  module Server
    class Socket

      include ::Upman::Utils::Helper
      include ::Upman::Utils::Dynload

      include WEBrick

      attr_reader :server

      def initialize
        @config = ::Upman::Core::Config.daemon

      end

      def run!

        server = HTTPServer.new(
            BindAddress: @config[:listen],
            :Port => @config[:port],
            :SSLEnable => @config[:ssl],
            :debugger => false,
            :daemonize => true,
            :SSLPrivateKey => OpenSSL::PKey::RSA.new(
                File.open(@config[:key]).read),
            :SSLCertificate => OpenSSL::X509::Certificate.new(
                File.open(@config[:cert]).read),
            :SSLCertName => [["CN", WEBrick::Utils::getservername]],
            Logger: WEBrick::Log.new("/dev/null"),
            AccessLog: [],
        )

        _register_extensions(server)

        protocol = @config[:ssl] ? 'https://' : 'http://'
        info "Starting listener on #{protocol}#{@config[:listen]}:#{@config[:port]}"
        server.start

      end

      private


      def _register_extensions(server)

        @config[:extensions].each do |extension|
          require_relative "../../upman/extensions/#{extension}"
          ext_obj  = dynload("Upman::Extensions::#{extension.split('_').map(&:capitalize).join('')}")
          info "Register /#{ext_obj ::REQUEST_PATH}.do for Upman::Extensions::#{extension.split('_').map(&:capitalize).join('')}"
          ext_class = ext_obj .new
          server.mount "/#{ext_obj ::REQUEST_PATH}.do", ext_class.register
        end
      end
    end
  end
end
